// IceWmMenuConfig.cpp Program to config menu of IceWm.
// 		       Written by Sombat C.    sombatsombat@yahoo.com
//		       30/12/2002
//                     required QT-X11-free-3.1.0
//		       Any changes welcome :)
//                     Last Changes 25-03-2003
// Changes: version 1.1 25-03-2003 : Drag and drop feature

#include <qapplication.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qobject.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qframe.h>
#include <qmessagebox.h>
#include <qvariant.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <stdio.h>
#include "IceWmMenuConfig.h"

QPixmap *menu_pixmap;
QPixmap *menu_drop_pixmap;
QPixmap *root_pixmap;
QPixmap *prog_pixmap;
QPixmap *prog_drop_pixmap;
QPixmap *restart_pixmap;
QPixmap *runonce_pixmap;
QPixmap *key_pixmap;
QPixmap *null_pixmap;
QPixmap *separator_pixmap;
QPixmap *menufile_pixmap;
QPixmap *include_pixmap;

char menufilename[256];
char icon_path[256];

enum menuNo { MenuAddSub=0,MenuAddProg,MenuAddRestart,MenuAddRunOnce,MenuAddKey,
              MenuAddSeparator,MenuEdit,MenuDelete,MenuCut,MenuPasteChild, 
	      MenuPasteElderBrother ,MenuPasteYoungerBrother};
enum listItemType { LIT_root=0,LIT_menu,LIT_prog,LIT_restart,LIT_runonce,LIT_key,LIT_separator,
		LIT_menufile, LIT_include};

enum itemColumn { TitleCol=0,IconCol,ProgCol,ArgumentCol }; 

char *parseIncludeStatement(char *p); 
static char *getCommandArgs(char *p, char *command, int command_len, char *args);
char *getWord(char *word, int maxlen, char *p);
char *parseMenus(char *data, IceWmMenuListViewItem *lv); 
int loadMenus(const char* menufile,IceWmMenuListViewItem *lv);
char *getArgument(char *dest, int maxLen, char *p, bool comma);
void add_path_icons(char *p);
void saveToFile(FILE *fp,IceWmMenuListViewItem *lv,int indent);
 
IceWmMenuListView :: IceWmMenuListView ( QWidget *parent )
    : QListView ( parent )
{
 root_lvi = new IceWmMenuListViewItem(this,LIT_root);
 addColumn("Title");
 addColumn("Icon");
 addColumn("Program");
 addColumn("Argument");
 setRootIsDecorated(TRUE);
 setSorting(-1);
 setPaletteBackgroundColor( QColor(170,255,255));
 menu = new QPopupMenu(this);
 menu->insertItem( QString( "Add SubMenu"), this, SLOT(slotAddMenuItem()),0,MenuAddSub);
 menu->insertItem( QString( "Add Program"), this, SLOT(slotAddProgItem(int)),0,MenuAddProg);
 menu->insertItem( QString( "Add Restart"), this, SLOT(slotAddProgItem(int)),0,MenuAddRestart);
 menu->insertItem( QString( "Add RunOnce"), this, SLOT(slotAddProgItem(int)),0,MenuAddRunOnce);
 menu->insertItem( QString( "Add Key??"), this, SLOT(slotAddProgItem(int)),0,MenuAddKey);
 menu->insertItem( QString( "Add Separator"), this, SLOT(slotAddSeparatorItem()),0,MenuAddSeparator);
 menu->insertItem( QString( "Edit"), this, SLOT(slotEditItem()),0,MenuEdit);
 menu->insertItem( QString( "Delete"), this, SLOT(slotDeleteItem()),0,MenuDelete);
 menu->insertItem( QString( "Cut"), this, SLOT(slotCutItem()),0,MenuCut);
 menu->insertItem( QString( "Paste as child"), this, SLOT(slotPasteAsChildItem()),0,MenuPasteChild);
 menu->insertItem( QString( "Paste as elder brother"), this, SLOT(slotPasteAsElderBrotherItem()),0,MenuPasteElderBrother);
 menu->insertItem( QString( "Paste as younger brother"), this, SLOT(slotPasteAsYoungerBrotherItem()),0,MenuPasteYoungerBrother);
/* using one slot for multiple choice, by passing integer to discriminate each one */
 menu->setItemParameter ( MenuAddProg, LIT_prog );
 menu->setItemParameter ( MenuAddRestart, LIT_restart );
 menu->setItemParameter ( MenuAddRunOnce, LIT_runonce );
 menu->setItemParameter ( MenuAddKey, LIT_key );
 
 connect(this, SIGNAL( contextMenuRequested (QListViewItem *, 
	const QPoint&, int)),this, SLOT( slotMenuRequested( QListViewItem *,
	const QPoint&, int)));

 connect(this, SIGNAL( mouseButtonPressed ( int , QListViewItem * , const QPoint & , int )),
	this, SLOT ( slotmouseButtonPressed( int, QListViewItem * , const QPoint &, int )));
 connect(this, SIGNAL( doubleClicked (QListViewItem *)), this, SLOT ( slotDoubleClickedEditItem(QListViewItem *) ));
 itemTaken=NULL;
 dirty=0;
 setAcceptDrops(TRUE);

}

void IceWmMenuListView :: slotAddMenuItem()
{
 if( ((IceWmMenuListViewItem *) selectedItem() )->item_type == LIT_root ||
     ((IceWmMenuListViewItem *) selectedItem() )->item_type == LIT_menu	){
	IceWmMenuListViewItem *lv = new IceWmMenuListViewItem(selectedItem(),LIT_menu);
	if (!lv) return;
	if (lv->title.length() != 0 ) {
		selectedItem()->setOpen(TRUE);
		moveLast(lv);
		dirty=1;
	}
	else
		delete(lv);
 }
 else
	// Program can't go here because menu was disabled for program line already.
	cout << "Can't add item to program line\n";
}

void IceWmMenuListView :: moveLast (IceWmMenuListViewItem *lv)
{
 if (lv->nextSibling()){
 	IceWmMenuListViewItem *lv2;
 	lv2=(IceWmMenuListViewItem *)lv->nextSibling();
	 //      try to set lv2 to the last child of lv's parent
	 while (lv2->nextSibling() != NULL){
	 	lv2 = (IceWmMenuListViewItem*)lv2->nextSibling();
	 }
	 //	move lv to be younger brother of lv2;
	 lv->moveItem(lv2);
 }
}

void IceWmMenuListView :: slotAddProgItem(int f_item_type)
{
 if( ((IceWmMenuListViewItem *)selectedItem())->item_type == LIT_root ||
     ((IceWmMenuListViewItem *)selectedItem())->item_type == LIT_menu	){
	IceWmMenuListViewItem *lv = new IceWmMenuListViewItem( selectedItem(),f_item_type);
	if (!lv) return;
	if (lv->title.length() == 0 || lv->icon.length() ==0 || lv->prog.length()==0 ) delete(lv);
	else{
		selectedItem()->setOpen(TRUE);
		moveLast(lv);
		dirty=1;
	}
 }
 else
	// Program can't go here because menu was disabled for program line already.
	cout << "Can't add item to program line\n";
}

void IceWmMenuListView :: slotAddSeparatorItem()
{
 if( ((IceWmMenuListViewItem *) selectedItem() )->item_type == LIT_root ||
     ((IceWmMenuListViewItem *) selectedItem() )->item_type == LIT_menu	){
	IceWmMenuListViewItem *lv = new IceWmMenuListViewItem(selectedItem(),LIT_separator);
	if (!lv) return;
	selectedItem()->setOpen(TRUE);
	moveLast(lv);
	dirty=1;
 }
 else
	// Program can't go here because menu was disabled for program line already.
	cout << "Can't add item to program line\n";
}

void IceWmMenuListView :: slotEditItem()
{
 IceWmMenuListViewItem *lv;
 lv=(IceWmMenuListViewItem *)selectedItem();
 if( lv->item_type == LIT_prog || lv->item_type == LIT_restart || lv->item_type == LIT_key ||
     lv->item_type == LIT_runonce ){
	progItemStructure pi;
	ProgDialog *progDialogInstance;
	pi.title=lv->title; 
	pi.icon=lv->icon; 
	pi.prog=lv->prog; 
	pi.argu=lv->argu; 
	if (lv->item_type == LIT_runonce) pi.winclass=lv->winclass;

	progDialogInstance = new ProgDialog ( &pi, lv->item_type,(QWidget*)this,"progitem",TRUE,
				Qt::WDestructiveClose | Qt::WType_Dialog | Qt::WShowModal );
	progDialogInstance->exec();
	if( progDialogInstance->result() == QDialog::Accepted ){
 		lv->setText(TitleCol,pi.title);

        	QFileInfo fi( pi.prog );
        	QString f_name = fi.fileName();
 		lv->setText(ProgCol, f_name);
// 		lv->setText(ProgCol, pi.prog);
		lv->setText(ArgumentCol, pi.argu);
		lv->title = pi.title;
		lv->icon = pi.icon;
		lv->prog = pi.prog;
		lv->argu = pi.argu;
		if (lv->item_type == LIT_runonce) lv->winclass= pi.winclass;
 		QFileInfo a ( pi.icon );
		if ( a.exists() )
			lv->setPixmap(IconCol,* new QPixmap ( pi.icon) );
		else
			lv->setPixmap(IconCol,*null_pixmap);
		dirty=1;
	}
 }
 else if ( lv->item_type == LIT_menu ){
	menuItemStructure mi;
	MenuDialog *menu_name_instance; 
	mi.title = lv->title;
	mi.icon = lv->icon;
	menu_name_instance = new MenuDialog ( &mi, (QWidget*)this,"menuname",TRUE, Qt::WDestructiveClose | Qt::WShowModal );
	menu_name_instance->exec();
	if( menu_name_instance->result() == QDialog::Accepted ){
 		lv->setText(TitleCol,mi.title );
		lv->title = mi.title;
		lv->icon = mi.icon;
 		QFileInfo a ( mi.icon );
		if ( a.exists() )
			lv->setPixmap(IconCol,* new QPixmap ( mi.icon) );
		else
			lv->setPixmap(IconCol,*null_pixmap);
		dirty=1;
	}
	// need to delete menu_name_instance?????
 }
 else 
	cout << "IceWm Menu Config: tried to edit unsupported item.\n";
}

void IceWmMenuListView :: slotDeleteItem()
{
	delete (selectedItem());
	dirty=1;
}

void IceWmMenuListView :: slotCutItem()
{
	itemTaken = selectedItem();
//	((QListViewItem *)itemTaken)->takeItem(itemTaken ); Why not work?
}

void IceWmMenuListView :: slotPasteAsChildItem()
{
	if(itemTaken ){
		selectedItem()->insertItem(itemTaken);
		moveLast((IceWmMenuListViewItem*)itemTaken);
		itemTaken=NULL;
		dirty=1;
	}
}

void IceWmMenuListView :: slotPasteAsYoungerBrotherItem()
{
	if(itemTaken){
		itemTaken->moveItem(selectedItem());		
		itemTaken=NULL;
		dirty=1;
	}
}

void IceWmMenuListView :: slotPasteAsElderBrotherItem()
{
	if(itemTaken){
	   QListViewItem *a;
	   a=selectedItem()->parent()->firstChild();
		/* if the destinated point is the highest one in its hierarchy */
	   if( selectedItem() == a){
		a=selectedItem()->parent();
				/* tricky part, first move itemTaken to be someone's child.
				   then move to be the child ( and it will be the first child ) of
				   destinated hierarchy */
		selectedItem()->insertItem(itemTaken);
		a->insertItem(itemTaken);
	   }
	   else{
		while( a->nextSibling() != selectedItem() )
			a=a->nextSibling();
		itemTaken->moveItem(a);
	   }	
	   itemTaken=NULL;
	   dirty=1;
	}
}

void IceWmMenuListView :: slotMenuRequested( QListViewItem *item, const QPoint &point, int)
{
 if (item ){
	int item_type =((IceWmMenuListViewItem*)item)->item_type;  
	menu->setItemEnabled(MenuPasteElderBrother,FALSE);
	menu->setItemEnabled(MenuPasteYoungerBrother,FALSE);
	menu->setItemEnabled(MenuPasteChild,FALSE);
	if( itemTaken && itemTaken != item && ((IceWmMenuListViewItem*)item)->child_of(
					(IceWmMenuListViewItem *) itemTaken) == FALSE) {
	    if( item_type == LIT_root || item_type == LIT_menu )
		menu->setItemEnabled(MenuPasteChild,TRUE);
	    if( item_type != LIT_root) {
		menu->setItemEnabled(MenuPasteYoungerBrother,TRUE);
		menu->setItemEnabled(MenuPasteElderBrother,TRUE);
	    }
	}
	if( item_type == LIT_root || item_type == LIT_menu){
		menu->setItemEnabled(MenuAddSub,TRUE);
		menu->setItemEnabled(MenuAddProg,TRUE);
		menu->setItemEnabled(MenuAddRestart,TRUE);
		menu->setItemEnabled(MenuAddRunOnce,TRUE);
		menu->setItemEnabled(MenuAddKey,TRUE);
		menu->setItemEnabled(MenuAddSeparator,TRUE);
	}
	else{
		menu->setItemEnabled(MenuAddSub,FALSE);
		menu->setItemEnabled(MenuAddProg,FALSE);
		menu->setItemEnabled(MenuAddRestart,FALSE);
		menu->setItemEnabled(MenuAddRunOnce,FALSE);
		menu->setItemEnabled(MenuAddKey,FALSE);
		menu->setItemEnabled(MenuAddSeparator,FALSE);
	}
	if( item_type == LIT_root || item_type == LIT_separator || item_type == LIT_menufile 
		|| item_type == LIT_include )
		menu->setItemEnabled(MenuEdit,FALSE);
	else
		menu->setItemEnabled(MenuEdit,TRUE);

	menu->popup(point);
 }
}

void IceWmMenuListView :: slotDoubleClickedEditItem( QListViewItem *)
{
	selectedItem()->setOpen( !selectedItem()->isOpen() );
	slotEditItem();
}

void IceWmMenuListView :: slotExitButton( )
{
 if (dirty){
 	switch( QMessageBox::warning( 0, "IceWm Menu Config",
        	"Content changed, want to save before exit?\n\n",
	        "Yes",
	        "No", 0, 0, 1 ) ){
	    case 0: 
		cout << "User clicked save\n";
	        break;
	    case 1: 
		cout << "User clicked not save\n";
	        break;
	}
 }
 qApp->quit();

}

void IceWmMenuListView :: slotSaveButton( )
{
 FILE *fp;
 fp=fopen(menufilename,"w");
 IceWmMenuListViewItem *lv = (IceWmMenuListViewItem *)this->firstChild();
 saveToFile(fp,lv,0);
 fclose(fp);
 dirty=0;
}

void IceWmMenuListView :: slotmouseButtonPressed ( int button, QListViewItem * item, const QPoint & /* pos */, int /* c */)
{
// cout << "Mouse Button Pressed, button is " << button << "\n";
 int item_type =((IceWmMenuListViewItem*)item)->item_type;  
 if( button == 1 && (item_type == LIT_prog || item_type == LIT_menu) ){  // only left button will create drag event
    IceWmMenuItemDrag *iwmID = new IceWmMenuItemDrag( item, this, "Drag Copy" );
    iwmID->dragCopy();
 }

}

void IceWmMenuListView::dragMoveEvent( QDragMoveEvent * e )
{
	static IceWmMenuListViewItem *oldItem=0;
    // Check if you want the drag at e->pos()...
    // Give the user some feedback...
	QPoint bb( e->pos().x(), e->pos().y() - treeStepSize() ); /* is it right to use treeStepSize? */
	IceWmMenuListViewItem * dropItem = ( IceWmMenuListViewItem *) itemAt(bb);
	if ( dropItem && oldItem != dropItem ){
		cout << "Gonna drop at " << dropItem->title << "\n";
		if( oldItem ){
			if( oldItem->item_type == LIT_menu )
				oldItem->setPixmap(TitleCol,*menu_pixmap);
			else if (oldItem->item_type == LIT_prog)
				oldItem->setPixmap(TitleCol,*prog_pixmap);
		}
		if( dropItem->item_type == LIT_menu )
			dropItem->setPixmap(TitleCol,*menu_drop_pixmap);
		else if (dropItem->item_type == LIT_prog)
			dropItem->setPixmap(TitleCol,*prog_drop_pixmap);
		oldItem = dropItem;
	}
}


void IceWmMenuListView::dragEnterEvent( QDragEnterEvent *e )
{
    // Check if you want the drag...
    if ( IceWmMenuItemDrag::canDecode( e ) )
    {
	e->accept();
    }

	
    // Give the user some feedback...
}

void IceWmMenuListView::dragLeaveEvent( QDragLeaveEvent *)
{
}


void IceWmMenuListView::dropEvent( QDropEvent * e )
{

 // Try to decode to the data you understand...

    QString str;
    IceWmMenuListViewItem *dragItem;
    if ( IceWmMenuItemDrag::decode( e, &dragItem) ) {
	IceWmMenuListViewItem *dropItem;
	QPoint bb( e->pos().x(), e->pos().y() - treeStepSize()  ); /* is it right to use treeStepSize? */

	cout << "Drag: " << dragItem->title << "\n";
	dropItem = ( IceWmMenuListViewItem *) itemAt(bb);
	if ( !dropItem ){
		cout << "Drop site is null\n";
		return;
	}
        cout << "Drop: " << dropItem->title << "\n";
 	if( dropItem->item_type == LIT_menu )
 		dropItem->setPixmap(TitleCol,*menu_pixmap);
 	else if (dropItem->item_type == LIT_prog)
		dropItem->setPixmap(TitleCol,*prog_pixmap);
	if( dragItem == dropItem ){
		cout << "Drag and drop at the same item\n";
		return;
	} 
	if( dropItem->item_type != LIT_menu && dropItem->item_type != LIT_prog ){
		cout << "Only drop on menu or prog is allowed\n";
		return;
	}
	if( dropItem->child_of(dragItem) ){
		cout << "Can't drag menu to be itself's child\n";
		return;
	}
		/* always paste yonger brother */
		dragItem->moveItem(dropItem);		
		dirty=1;
/*	if( dropItem->item_type == LIT_menu ){
		if( dragItem->item_type == LIT_prog ){
			dropItem->insertItem(dragItem);
			moveLast(dragItem);
		}
		else{
			dragItem->moveItem(dropItem);		
		}
		dirty=1;
	}
	else{
		dragItem->moveItem(dropItem);		
		dirty=1;
	}		 */
	return;
    }

}

void IceWmMenuListView::mouseReleaseEvent ( QMouseEvent * /* e */) // why this function is not called?
{
// cout << "ListViewItem get dropped is at ptr " << itemAt( e->pos() ) << "\n";
} 

 
IceWmMenuItemDrag::IceWmMenuItemDrag( QListViewItem *item, QWidget * parent, const char * name )
    : QStoredDrag( "IceWmMenu/Item", parent, name )
{
 int ptrSize,i;
 unsigned char *ptr;

 ptr=(unsigned char*) &item;
 ptrSize = sizeof(item);
    QByteArray data(ptrSize);
    for( i = 0 ; i < ptrSize; i ++ ){
	data[i] = *ptr;
	ptr++;
    } 
    setEncodedData( data );
}


bool IceWmMenuItemDrag::canDecode( QDragMoveEvent* e )
{
    return e->provides( "IceWmMenu/Item" );
}

//decode it into a string
bool IceWmMenuItemDrag::decode( QDropEvent* e,IceWmMenuListViewItem ** dragItem  )
{
    unsigned char* ptr;
    QByteArray payload = e->data( "IceWmMenu/Item" );
    if ( payload.size() ) {
	e->accept();
	unsigned int i;
	ptr = (unsigned char *)dragItem;
	for(i=0; i<payload.size(); i++){
		*ptr = (unsigned char) (payload [i] & 0xff);
		ptr++;
	}
	return TRUE;
    }
    return FALSE;
}


void saveToFile(FILE *fp,IceWmMenuListViewItem *lv,int indent)
{
 IceWmMenuListViewItem *lv2;
 char icon[256],*p;
 int i;
 QString title;
 lv2=(IceWmMenuListViewItem*)lv->firstChild();
 while(lv2){
//	strip_path(lv2->icon,icon);
	if( lv2->icon.length() !=0){
		QFileInfo f(lv2->icon);
		strcpy(icon,f.fileName().ascii());
		p=strstr(icon,"_16x16");
		if(p)
			*p='\0';
	}
	else
		strcpy(icon,"");
	if ( lv2->title.length()!=0)
		// any white space in title? if so, put " " around it
		if ( strchr(lv2->title.ascii(),' ') )
			title='"' + lv2->title + '"';
		else
			title=lv2->title;
	else
		title="";

	switch(lv2->item_type){
		case LIT_menu:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"menu %s %s {\n",title.ascii(),icon);
			saveToFile(fp,lv2,indent+1);
			fprintf(fp,"}\n");
			break;
		case LIT_prog:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"prog %s %s %s %s\n",title.ascii(),icon,lv2->prog.ascii(),lv2->argu.ascii());
			break;
		case LIT_restart:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"restart %s %s %s\n",title.ascii(),icon,lv2->prog.ascii());
			break;
		case LIT_runonce:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"runonce %s %s %s %s %s\n",title.ascii(),icon,lv2->winclass.ascii(),lv2->prog.ascii(),lv2->argu.ascii());
			break;
		case LIT_key:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"prog %s %s %s %s\n",title.ascii(),icon,lv2->prog.ascii(),lv2->argu.ascii());
			break;
		case LIT_separator:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"separator\n");
			break;
		case LIT_menufile:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"menufile %s %s %s\n",title.ascii(),icon,lv2->prog.ascii());
			break;
		case LIT_include:
			for(i=0;i<indent;i++) fprintf(fp,"    ");
			fprintf(fp,"include %s\n",title.ascii());
			break;
		default:
			cout << "Who are you??\n";

	}
	lv2=(IceWmMenuListViewItem*)lv2->nextSibling();
 }
}

IceWmMenuListViewItem::IceWmMenuListViewItem( QListView *parent, int item_type_to_be_created)
 : QListViewItem (parent)
{
	item_type=item_type_to_be_created;
 	//setText(TitleCol,QString(""));

	setPixmap(TitleCol,*root_pixmap);
}
 
IceWmMenuListViewItem::IceWmMenuListViewItem( QListViewItem *parent, int item_type_to_be_created)
 : QListViewItem (parent)
{
 int f_item_type;
 f_item_type = item_type_to_be_created;
 switch ( f_item_type ){
   case LIT_menu :{
	menuItemStructure mi;
	MenuDialog *menuDialogInstance = new MenuDialog ( &mi, 0,"menuname",TRUE, Qt::WDestructiveClose | Qt::WShowModal );
	menuDialogInstance->exec();
	item_type=f_item_type;
	if( menuDialogInstance->result() == QDialog::Accepted ){
 		setText(TitleCol,mi.title);
		setPixmap(TitleCol,*menu_pixmap);
 		QFileInfo a ( mi.icon );
		if ( a.exists() )
			setPixmap(IconCol,* new QPixmap ( mi.icon) );
		else
			setPixmap(IconCol,*null_pixmap);
		title = mi.title;
		icon = mi.icon;
	} 
	}
	break;
   case LIT_prog :	
   case LIT_restart :	
   case LIT_runonce :
   case LIT_key :
	{
	progItemStructure pi;	
	ProgDialog *progDialogInstance = new ProgDialog ( &pi,f_item_type, 0,"progitem",TRUE,Qt::WDestructiveClose | Qt::WType_Dialog | Qt::WShowModal );
	progDialogInstance->exec();
	item_type=f_item_type;
	if( progDialogInstance->result() == QDialog::Accepted ){
 		setText(TitleCol,pi.title);
 		setText(ProgCol, pi.prog);
		setText(ArgumentCol, pi.argu);
		switch( f_item_type ){
			case LIT_prog:    setPixmap(TitleCol,*prog_pixmap); break;
			case LIT_restart: setPixmap(TitleCol,*restart_pixmap); break;
			case LIT_runonce: setPixmap(TitleCol,*runonce_pixmap);
					  winclass = pi.winclass;
 					  break;
			case LIT_key:	  setPixmap(TitleCol,*key_pixmap); break;
		}
 		QFileInfo a ( pi.icon );
		if ( a.exists() )
			setPixmap(IconCol,* new QPixmap ( pi.icon) );
		else
			setPixmap(IconCol,*null_pixmap);
		title = pi.title;
		icon = pi.icon;
		prog = pi.prog;
		argu = pi.argu;
	}
	}
	break;
   case LIT_separator :
	item_type=f_item_type;
	setPixmap( TitleCol,*separator_pixmap);
	break;
 }
}

IceWmMenuListViewItem::IceWmMenuListViewItem( QListViewItem *parent,progItemStructure *pi, int item_type_to_be_created)
 : QListViewItem (parent)
{
 int f_item_type;
 f_item_type = item_type_to_be_created;
 switch ( f_item_type ){
   case LIT_menu :{
	item_type=f_item_type;
 	setText(TitleCol,pi->title);
	setPixmap(TitleCol,*menu_pixmap);
 	QFileInfo a ( pi->icon );
	if ( a.exists() )
		setPixmap(IconCol,* new QPixmap ( pi->icon) );
	else
		setPixmap(IconCol,*null_pixmap);
	title = pi->title;
	icon = pi->icon;
	}
	break;
   case LIT_prog :	
   case LIT_restart :	
   case LIT_runonce :
   case LIT_key :
	{
	item_type=f_item_type;
 	setText(TitleCol,pi->title);
 	setText(ProgCol, pi->prog);
	setText(ArgumentCol, pi->argu);
	title = pi->title;
	icon = pi->icon;
	prog = pi->prog;
	argu = pi->argu;
	switch( f_item_type ){
		case LIT_prog:    setPixmap(TitleCol,*prog_pixmap); break;
		case LIT_restart: setPixmap(TitleCol,*restart_pixmap); break;
		case LIT_runonce: setPixmap(TitleCol,*runonce_pixmap);
				  winclass = pi->winclass;
 				  break;
		case LIT_key:	  setPixmap(TitleCol,*key_pixmap); break;
	}
 	QFileInfo a ( pi->icon );
	if ( a.exists() )
		setPixmap(IconCol,* new QPixmap ( pi->icon) );
	else
		setPixmap(IconCol,*null_pixmap);
	}
	break;
   case LIT_separator :
	item_type=f_item_type;
	setPixmap( TitleCol,*separator_pixmap);
	break;
   case LIT_menufile :
	item_type=f_item_type;
	title=pi->title;
	icon=pi->icon;
	prog=pi->prog;
	setPixmap( TitleCol,*menufile_pixmap);
	break;
   case LIT_include :
	item_type=f_item_type;
	title=pi->title;
	setPixmap( TitleCol,*include_pixmap);
	break;
 }
 moveLast(this);
}

QDragObject *IceWmMenuListViewItem :: dragObject()
{
 cout << "Hello World\n";
// QDragObject::QDragObject ( QWidget * dragSource = 0, const char * name = 0 )
 return 0;
}
QDragObject *IceWmMenuListView :: dragObject()
{
 cout << "Hello World2\n";
// QDragObject::QDragObject ( QWidget * dragSource = 0, const char * name = 0 )
 return 0;
}

void IceWmMenuListViewItem::mousePressEvent( QMouseEvent * /*e*/ )
{
/*    SecretDrag *sd = new SecretDrag( mySecret, this );
    sd->setPixmap(QPixmap(picture_xpm),QPoint(8,8));
    sd->dragCopy();
    mySecret++; */
	cout << "Mouse was pressed on List View Item\n";
}

bool IceWmMenuListViewItem :: child_of (IceWmMenuListViewItem *a)
{
	IceWmMenuListViewItem *b;
	b=this;
	if (b->item_type == LIT_root )
		return FALSE;
	while (((IceWmMenuListViewItem*)(b->parent()))->item_type != LIT_root){
		b = (IceWmMenuListViewItem*)(b->parent());
		if ( b == a )
			return TRUE;
	}
	return FALSE;
		 
}

void IceWmMenuListViewItem :: moveLast (IceWmMenuListViewItem *lv)
{
 if (lv->nextSibling()){
 	IceWmMenuListViewItem *lv2;
 	lv2=(IceWmMenuListViewItem *)lv->nextSibling();
	 //      try to set lv2 to the last child of lv's parent
	 while (lv2->nextSibling() != NULL){
	 	lv2 = (IceWmMenuListViewItem*)lv2->nextSibling();
	 }
	 //	move lv to be younger brother of lv2;
	 lv->moveItem(lv2);
 }
}
MenuDialog::MenuDialog( menuItemStructure *f_menuItemStructure,QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    miPtr = f_menuItemStructure;
    if ( !name )
	setName( "MenuDialog" );
    setPaletteBackgroundColor( QColor( 170, 255, 127 ) );
    setSizeGripEnabled( TRUE );

    textLabel1 = new QLabel( this, "textLabel1" );
    textLabel1->setGeometry( QRect( 20, 40, 110, 35 ) );
    QFont textLabel1_font(  textLabel1->font() );
    textLabel1_font.setFamily( "Helvetica [Adobe]" );
    textLabel1_font.setPointSize( 16 );
    textLabel1->setFont( textLabel1_font ); 

    iconLabel = new QLabel( this, "iconLabel" );
    iconLabel->setGeometry( QRect( 20, 100, 110, 35 ) );
    QFont iconLabel_font( iconLabel->font() );
    iconLabel_font.setFamily( "Helvetica [Adobe]" );
    iconLabel_font.setPointSize( 16 );
    iconLabel->setFont( iconLabel_font );  

    QWidget* privateLayoutWidget = new QWidget( this, "Layout1" );
    privateLayoutWidget->setGeometry( QRect( 110, 160, 230, 33 ) );
    Layout1 = new QHBoxLayout( privateLayoutWidget, 0, 50, "Layout1"); 

    buttonOk = new QPushButton( privateLayoutWidget, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
    Layout1->addWidget( buttonOk );

    buttonCancel = new QPushButton( privateLayoutWidget, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
    Layout1->addWidget( buttonCancel );

    textEdit1 = new QTextEdit( this, "textEdit1" );
    textEdit1->setGeometry( QRect( 140, 40, 200, 35 ) );
    textEdit1->setTabChangesFocus ( TRUE);
    QFont textEdit1_font (  textEdit1->font() );
    textEdit1_font.setFamily( "Helvetica [Adobe]" );
    textEdit1_font.setPointSize( 16 );
    textEdit1->setFont (textEdit1_font);
    textEdit1->setVScrollBarMode ( QTextEdit::AlwaysOff );
    textEdit1->setHScrollBarMode ( QTextEdit::AlwaysOff );

    iconEdit = new QTextEdit( this, "iconEdit" );
    iconEdit->setGeometry( QRect( 140, 100, 200, 35 ) );
    iconEdit->setTabChangesFocus ( TRUE);
    QFont iconEdit_font (  iconEdit->font() );
    iconEdit_font.setFamily( "Helvetica [Adobe]" );
    iconEdit_font.setPointSize( 16 );
    iconEdit->setFont (iconEdit_font);
    iconEdit->setVScrollBarMode ( QTextEdit::AlwaysOff );
    iconEdit->setHScrollBarMode ( QTextEdit::AlwaysOff );

    iconBrowse = new QPushButton (this, "iconBrowse");
    iconBrowse->setGeometry ( QRect (350,100,30,35));

    languageChange();
    resize( QSize(435, 223).expandedTo(minimumSizeHint()) );
    textEdit1->setFocus();

    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT(returnMenuItem() ) );
    connect( iconBrowse, SIGNAL( clicked() ), this, SLOT(get_icon_name() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
MenuDialog::~MenuDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void MenuDialog::languageChange()
{
    setCaption( tr( "MenuDialog" ) );
    textLabel1->setText( tr("Menu name" ) );
    iconLabel->setText( "Icon (.xpm)");
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAccel( QKeySequence( QString::null ) );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAccel( QKeySequence( QString::null ) );
    iconBrowse->setText( tr ("..."));
    textEdit1->setText( miPtr->title);
    iconEdit->setText (miPtr->icon);
}

void MenuDialog::returnMenuItem()
{
   if( textEdit1->text().length() !=0 ){
       	miPtr->title = textEdit1->text();
	miPtr->icon = iconEdit->text();
	accept();
   }
   else{
	fputc(0x07,stdout);
	fflush(stdout);
   }
   
}

void MenuDialog::get_icon_name()
{
 QFileDialog* fd = new QFileDialog( this, "get icon name", TRUE );
 fd->setMode( QFileDialog::ExistingFile );
 QFileInfo a ( miPtr->icon );
 fd->setFilter( "Images (*.xpm)" );
 if ( a.exists() ){
	fd->setSelection ( miPtr->icon);
 }
 else
 	fd->setDir (icon_path );

 Preview* p = new Preview;

 fd->setContentsPreviewEnabled( TRUE );
 fd->setContentsPreview( p, p );
 fd->setPreviewMode( QFileDialog::Contents );

 if ( fd->exec() == QDialog::Accepted )
     iconEdit->setText(fd->selectedFile());
     
}

ProgDialog::ProgDialog( progItemStructure *f_progItemStructure,int f_item_type,QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    piPtr = f_progItemStructure;
    item_type = f_item_type;
    if ( !name )
	setName( "ProgItem" );
    setPaletteBackgroundColor( QColor( 170, 255, 127 ) );

    textLabel4 = new QLabel( this, "textLabel4" );
    textLabel4->setGeometry( QRect( 40, 150, 61, 20 ) );
    textLabel4->setFrameShape( QLabel::NoFrame );
    textLabel4->setFrameShadow( QLabel::Plain );

    textLabel3 = new QLabel( this, "textLabel3" );
    textLabel3->setGeometry( QRect( 40, 110, 61, 20 ) );

    textLabel1 = new QLabel( this, "textLabel1" );
    textLabel1->setGeometry( QRect( 40, 30, 61, 20 ) );

    textLabel2 = new QLabel( this, "textLabel2" );
    textLabel2->setGeometry( QRect( 40, 70, 61, 20 ) );

    cancelButton = new QPushButton( this, "cancelButton" );
    cancelButton->setGeometry( QRect( 250, 200, 85, 30 ) );

    okButton = new QPushButton( this, "okButton" );
    okButton->setGeometry( QRect( 100, 200, 85, 30 ) );

    progBrowseButton = new QPushButton( this, "progBrowseButton" );
    progBrowseButton->setGeometry( QRect( 340, 100, 30, 30 ) );

    iconText = new QTextEdit( this, "iconText" );
    iconText->setGeometry( QRect( 130, 60, 200, 30 ) );
    iconText->setTabChangesFocus( TRUE );
    iconText->setVScrollBarMode ( QTextEdit::AlwaysOff);
    iconText->setHScrollBarMode ( QTextEdit::AlwaysOff);

    progText = new QTextEdit( this, "progText" );
    progText->setGeometry( QRect( 130, 100, 200, 30 ) );
    progText->setTabChangesFocus( TRUE );
    progText->setVScrollBarMode ( QTextEdit::AlwaysOff);
    progText->setHScrollBarMode ( QTextEdit::AlwaysOff);

    argumentText = new QTextEdit( this, "argumentText" );
    argumentText->setGeometry( QRect( 130, 140, 200, 30 ) );
    argumentText->setTabChangesFocus( TRUE );
    argumentText->setVScrollBarMode ( QTextEdit::AlwaysOff);
    argumentText->setHScrollBarMode ( QTextEdit::AlwaysOff);
    titleText = new QTextEdit( this, "titleText" );
    titleText->setGeometry( QRect( 130, 20, 200, 30 ) );
    titleText->setTabChangesFocus( TRUE );
    titleText->setVScrollBarMode ( QTextEdit::AlwaysOff);
    titleText->setHScrollBarMode ( QTextEdit::AlwaysOff);

    iconBrowseButton = new QPushButton( this, "iconBrowseButton" );
    iconBrowseButton->setGeometry( QRect( 340, 60, 30, 30 ) );
    resize( QSize(415, 270).expandedTo(minimumSizeHint()) );
  if( item_type == LIT_runonce ){
    textLabel5 = new QLabel( this, "textLabel5" );
    textLabel5->setGeometry( QRect( 40, 190, 61, 20 ) );
    textLabel5->setFrameShape( QLabel::NoFrame );
    textLabel5->setFrameShadow( QLabel::Plain );

    classText = new QTextEdit( this, "classText" );
    classText->setGeometry( QRect( 130, 190, 200, 30 ) );
    classText->setTabChangesFocus( TRUE );
    classText->setVScrollBarMode ( QTextEdit::AlwaysOff);
    classText->setHScrollBarMode ( QTextEdit::AlwaysOff);
    
    cancelButton->setGeometry( QRect( 270, 240, 85, 30 ) );
    okButton->setGeometry( QRect( 120, 240, 85, 30 ) );
    resize( QSize(415, 315).expandedTo(minimumSizeHint()) );
  }
    languageChange();


    // signals and slots connections
    connect( cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( iconBrowseButton, SIGNAL( clicked() ), this, SLOT( get_icon_name() ) );
    connect( progBrowseButton, SIGNAL( clicked() ), this, SLOT( get_prog_name() ) ); 
    connect( okButton, SIGNAL( clicked() ), this, SLOT( returnProgItem() ) );

    // tab order
    setTabOrder( titleText, iconText );
    setTabOrder( iconText, progText );
    setTabOrder( progText, argumentText );
    setTabOrder( argumentText, okButton );
    setTabOrder( okButton, cancelButton );
    setTabOrder( cancelButton, iconBrowseButton );
    setTabOrder( iconBrowseButton, progBrowseButton );
}

/*
 *  Destroys the object and frees any allocated resources
 */
ProgDialog::~ProgDialog()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void ProgDialog::languageChange()
{
    setCaption( tr( "ProgItem" ) );
    textLabel4->setText( tr( "Arguments" ) );
    textLabel3->setText( tr( "Program" ) );
    textLabel1->setText( tr( "Title" ) );
    textLabel2->setText( tr( "Icon (.xpm)" ) );
    cancelButton->setText( tr( "Cancel" ) );
    progBrowseButton->setText( tr( "..." ) );
    okButton->setText( tr( "Ok" ) );
    iconBrowseButton->setText( tr( "..." ) );
    titleText->setText(piPtr->title);
    iconText->setText(piPtr->icon);
    progText->setText(piPtr->prog);
    argumentText->setText(piPtr->argu);
  if( item_type == LIT_runonce ){
    textLabel5->setText( tr( "Win class" ) );
    classText->setText(piPtr->winclass);
  }
}

void ProgDialog::returnProgItem()
{
 if( titleText->text().length() == 0 ||
     iconText->text().length() == 0  ||
     progText->text().length() == 0  ){
    fputc(0x07,stdout);
    fflush(stdout);
    return;
 }
 piPtr->title = titleText->text();
 piPtr->icon  = iconText->text();
 piPtr->prog  = progText->text();
 piPtr->argu  = argumentText->text();
 if( item_type == LIT_runonce )
     piPtr->winclass  = classText->text();
 accept();
}

void ProgDialog::get_icon_name()
{
 QFileDialog* fd = new QFileDialog( this, "get icon name", TRUE );
 fd->setMode( QFileDialog::ExistingFile );
 QFileInfo a ( piPtr->icon );
 fd->setFilter( "Images (*.xpm)" );
 if ( a.exists() ){
	fd->setSelection ( piPtr->icon);
 }
 else
 	fd->setDir (icon_path  );

 Preview* p = new Preview;

 fd->setContentsPreviewEnabled( TRUE );
 fd->setContentsPreview( p, p );
 fd->setPreviewMode( QFileDialog::Contents );

 if ( fd->exec() == QDialog::Accepted )
     iconText->setText(fd->selectedFile());
     
}

void ProgDialog::get_prog_name()
{
 progText->setText(QFileDialog::getOpenFileName( "/usr/bin/",
	"*", this, "get prog name", "Choose the program to be executed") );
}
 
int main(int argc,char **argv)
{
 QApplication a( argc, argv);
 
 QSplitter *sp = new QSplitter (Qt::Vertical,0);

// sp->setPaletteBackgroundColor( QColor(170,255,255));
 sp->setGeometry( QRect (60,60,670,450));
 menu_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/menu.xpm");
 menu_drop_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/menu_drop.xpm");
 root_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/start.xpm");
 prog_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/prog.xpm");
 prog_drop_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/prog_drop.xpm");
 restart_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/restart.xpm");
 runonce_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/runonce.xpm");
 key_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/key.xpm");
 null_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/null.xpm");
 separator_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/separator.xpm");
 menufile_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/menufile.xpm");
 include_pixmap = new QPixmap("/usr/local/IceWmMenuConfig/include.xpm");

 IceWmMenuListView *lv = new IceWmMenuListView (sp);
 QFrame* btnPanel = new QFrame(sp);
  
 QPushButton *saveButton = new QPushButton( btnPanel, "saveButton" );
 saveButton->setGeometry( QRect( 170, 5, 85, 30 ) );
 saveButton->setText( "Save" );

 QPushButton *exitButton = new QPushButton( btnPanel, "exitButton" );
 exitButton->setGeometry( QRect( 385, 5, 85, 30 ) );
 exitButton->setText( "Exit" );

 QPushButton::connect( exitButton, SIGNAL( clicked()), lv, SLOT( slotExitButton()) );
 QPushButton::connect( saveButton, SIGNAL( clicked()), lv, SLOT( slotSaveButton()) );

 strcpy(menufilename,getenv("HOME"));
 strcat(menufilename,"/.icewm/menu"); 
// strcpy(menufilename,"/tmp/menu");
 if(getenv("ICEWM_MENU_CONFIG_ICON_PATH"))
	 strcpy(icon_path,getenv("ICEWM_MENU_CONFIG_ICON_PATH"));
 if( strlen(icon_path)==0)
//	strcpy(icon_path,"/usr/local/lib/X11/icewm/icons/");
	strcpy(icon_path,"/usr/local/share/icewm/icons/");
 loadMenus(menufilename,(IceWmMenuListViewItem*)lv->firstChild());
 a.setMainWidget( sp );
 lv->firstChild()->setOpen(TRUE);
 sp->show();

 return a.exec();
}

int loadMenus(const char* menufile,IceWmMenuListViewItem *lv)
{
 QFileInfo f (menufile);
 FILE *fp;
 if( !f.exists()){
	cout << "Menu file: '" << menufile << "' not exist\n";
	return 1;
 }
 fp = fopen( menufile, "r");
 if ( !fp ){
	cout << "Error opening menu file: '" << menufile << "\n";
	return 2;
 }
 if( f.size()<1 ){
	cout << "Error menu file size: '" << menufile << "\n";
	fclose(fp);
	return 3;
 }
 char *buf = new char[f.size()];
 if( buf == 0){
	cout << "Error allocating buffer for menufile\n";
	fclose(fp);
	return 4;
 }
 fread(buf,f.size(), 1,fp);
 fclose(fp); 

 parseMenus(buf,lv);

 delete[] buf;
 return 0;
} 

char *parseMenus(char *data, IceWmMenuListViewItem *lv) 
{
    char *p = data;
    char word[32];
    progItemStructure pi;

    while (p && *p) {
        while (*p == ' ' || *p == '\t' || *p == '\n')
            p++;
        if (*p == '#') {
            while (*p && *p != '\n')
                p++;
            continue;
        }
        p = getWord(word, sizeof(word), p);

        if (lv) {
	    if (!strcmp(word, "separator"))
		(void ) new IceWmMenuListViewItem( lv,&pi,LIT_separator);
	    else if (!(strcmp(word, "prog") &&
		       strcmp(word, "restart") &&
		       strcmp(word, "runonce"))) {
		char name[64];

		p = getArgument(name, sizeof(name), p, false);
		if (p == 0) return p;

		char icons[128];

		p = getArgument(icons, sizeof(icons), p, false);
		if (p == 0) return p;
		add_path_icons(icons);

		char wmclass[256];

		if (word[1] == 'u') {
		    p = getArgument(wmclass, sizeof(wmclass), p, false);
		    if (p == 0) return p;
		}

		char command[256];
		char args[256];
		args[0]='\0';

		p = getCommandArgs(p, command, sizeof(command), args);
		if (p == 0) {
		    cout << "Error at prog " << name; return p;
		}
		pi.title=name;
		pi.icon=icons;
		pi.prog=command;
		pi.argu=args;
		if( !strcmp(word,"prog") )
			(void) new IceWmMenuListViewItem( lv,&pi,LIT_prog);
		else if( !strcmp(word,"restart") )
			(void) new IceWmMenuListViewItem( lv,&pi,LIT_restart);
		else if( !strcmp(word,"runonce") ){
			pi.winclass = wmclass;
			(void) new IceWmMenuListViewItem( lv,&pi,LIT_runonce);
		}
	    } else if (!strcmp(word, "menu")) {
		char name[64];

		p = getArgument(name, sizeof(name), p, false);
		if (p == 0) return p;

		char icons[128];

		p = getArgument(icons, sizeof(icons), p, false);
		if (p == 0) return p;
		add_path_icons(icons);

		p = getWord(word, sizeof(word), p);
		if (*p != '{') return 0;
		p++;

		pi.title=name;
		pi.icon=icons;
		IceWmMenuListViewItem *lv1 = new IceWmMenuListViewItem( lv, &pi, LIT_menu);
		if (lv1) {
		    p = parseMenus(p, lv1);
		} else {
                    cout << "Unexepected keyword: " << word << "\n";
		    return p;
                }
	    } else if (!strcmp(word, "menufile")) {
		char name[64];

		p = getArgument(name, sizeof(name), p, false);
		if (p == 0) return p;

		char icons[128];

		p = getArgument(icons, sizeof(icons), p, false);
		if (p == 0) return p;
		add_path_icons(icons);

                char menufile[128];

		p = getArgument(menufile, sizeof(menufile), p, false);
                if (p == 0) return p;

		pi.title=name;
		pi.icon=icons;
		pi.prog=menufile;
		(void) new IceWmMenuListViewItem( lv,&pi,LIT_menufile);
		cout << "menufile keyword is not yet implemented!\n";
	

            } else if (!strcmp(word, "include")){
    		char filename[256];
    		p = getArgument(filename, sizeof(filename), p, false);
		pi.title=filename;
		(void) new IceWmMenuListViewItem( lv,&pi,LIT_include);
		cout << "include keyword is not yet implemented!\n";
	    }
	    else if (*p == '}')
		return ++p;
	    else {
		return 0;
            }
        } else {
	    cout << "Ever get to this point???\n";
	    if (!(strcmp(word, "key") &&
	          strcmp(word, "runonce"))) {
		char key[64];

		p = getArgument(key, sizeof(key), p, false);
		if (p == 0) return p;

		char wmclass[256];

		if (*word == 'r') {
		    p = getArgument(wmclass, sizeof(wmclass), p, false);
		    if (p == 0) return p;
		}

		char command[256];
		char args[256];

		p = getCommandArgs(p, command, sizeof(command), args);
		if (p == 0) {
		    cout << "Error at key " << key << "\n";
		    return p;
		}

            } else {
		return 0;
            }
        }
    }
    return p;
}

char *getWord(char *word, int maxlen, char *p) {
    while (*p && (*p == ' ' || *p == '\t' || *p == '\r' || *p == '\n'))
        p++;
    while (*p && isalnum(*p) && maxlen > 1) {
        *word++ = *p++;
        maxlen--;
    }
    *word++ = 0;
    return p;
}

static char *getCommandArgs(char *p, char *command, int command_len,
                     	    char *args) {
    p = getArgument(command, command_len, p, false);
    if (p == 0) {
        cout << "Missing command argument\n";
        return p;
    }

    while (*p) {
        char argx[256];

        while (*p && (*p == ' ' || *p == '\t'))
            p++;

        if (*p == '\n')
            break;

        p = getArgument(argx, sizeof(argx), p, false);
        if (p == 0) {
            cout << "Bad argument\n";
            return p;
        }
	strcat(args,argx);
	strcat(args," ");
    }
	/* last character of args is " " --> need to be truncate? */
    return p;
}

char *getArgument(char *dest, int maxLen, char *p, bool comma) {
    char *d;
    int len = 0;
    int in_str = 0;
    
    while (*p && (*p == ' ' || *p == '\t'))
        p++;

    d = dest;
    len = 0;
    while (*p && len < maxLen - 1 &&
           (in_str || (*p != ' ' && *p != '\t' && *p != '\n' && (!comma || *p != ','))))
    {
        if (in_str && *p == '\\' && p[1]) {
            p++; char c = *p++; // *++p++ doesn't work :(

            switch (c) {
            case 'a': *d++ = '\a'; break;
            case 'b': *d++ = '\b'; break;
            case 'e': *d++ = 27; break;
            case 'f': *d++ = '\f'; break;
            case 'n': *d++ = '\n'; break;
            case 'r': *d++ = '\r'; break;
            case 't': *d++ = '\t'; break;
            case 'v': *d++ = '\v'; break;
            case 'x':
#define UNHEX(c) \
    (\
    ((c) >= '0' && (c) <= '9') ? (c) - '0' : \
    ((c) >= 'A' && (c) <= 'F') ? (c) - 'A' + 0xA : \
    ((c) >= 'a' && (c) <= 'f') ? (c) - 'a' + 0xA : 0 \
    )
                if (p[0] && p[1]) { // only two digits taken
                    int a = UNHEX(p[0]);
                    int b = UNHEX(p[1]);

                    int n = (a << 4) + b;

                    p += 2;
                    *d++ = (unsigned char)(n & 0xFF);

                    a -= '0';
                    if (a > '9')
                        a = a + '0' - 'A';
                    break;
                }
            default:
                *d++ = c;
                break;
            }
            len++;
        } else if (*p == '"') {
            in_str = !in_str;
            p++;
        } else {
            *d++ = *p++;
            len++;
        }
    }
    *d = 0;

    return p;
}

char *parseIncludeStatement(char *p) 
{
    char filename[256];

    p = getArgument(filename, sizeof(filename), p, false);

    return p;
}

void add_path_icons(char *p)
{
 QString a;

 a=icon_path;
 a=a+p+"_16x16.xpm";
 strcpy(p,a.ascii());
}
