#ifndef IceWmMenuConfig_H
#define IceWmMenuConfig_H
#include <qlistview.h>
#include <qobject.h>
#include <qpopupmenu.h>
#include <qvariant.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <qcstring.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QLabel;
class QPushButton;
class QTextEdit;

struct menuItemStructure {
	QString title;
	QString icon;
};

class MenuDialog : public QDialog
{
    Q_OBJECT

public:
    MenuDialog( menuItemStructure *f_menuItemStructure,QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~MenuDialog();

    QLabel* textLabel1;
    QTextEdit* textEdit1;
    QLabel* iconLabel;
    QTextEdit* iconEdit;
    QPushButton* buttonOk;
    QPushButton* buttonCancel;
    QPushButton* iconBrowse;

private:
    menuItemStructure *miPtr;

protected:
    QHBoxLayout* Layout1;

protected slots:
    virtual void languageChange();
    void returnMenuItem();
    void get_icon_name();
};


struct progItemStructure {
	QString title;
	QString icon;
	QString prog;
	QString argu;
	QString winclass;
};


class ProgDialog : public QDialog
{
    Q_OBJECT

public:
    ProgDialog( progItemStructure *f_progItemStructure,int f_item_type,QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~ProgDialog();

    QLabel* textLabel4;
    QLabel* textLabel5;
    QLabel* textLabel3;
    QLabel* textLabel1;
    QLabel* textLabel2;
    QPushButton* cancelButton;
    QPushButton* progBrowseButton;
    QTextEdit* iconText;
    QTextEdit* progText;
    QTextEdit* argumentText;
    QTextEdit* classText;
    QPushButton* okButton;
    QTextEdit* titleText;
    QPushButton* iconBrowseButton;

private:
    progItemStructure *piPtr;
    int item_type;


protected:

protected slots:
    virtual void languageChange();
    void returnProgItem();
    void get_icon_name();
    void get_prog_name();
};

class Preview : public QLabel, public QFilePreview
{
 public:
    Preview( QWidget *parent=0 ) : QLabel( parent ) {}

    void previewUrl( const QUrl &u )
    {
        QString path = u.path();
        QPixmap pix( path );
        if ( pix.isNull() )
            setText( "This is not a pixmap" );
        else
            setPixmap( pix );
    }
};


class IceWmMenuListViewItem : public QListViewItem
{
public: IceWmMenuListViewItem ( QListView *parent,int item_type_to_be_created);
	IceWmMenuListViewItem ( QListViewItem *parent, int item_type_to_be_created);
	IceWmMenuListViewItem ( QListViewItem *parent,progItemStructure *pi, int item_type_to_be_created);
	QDragObject *dragObject();
	int item_type;
	QString title;
	QString icon;
	QString prog;
	QString argu;
	QString winclass;
	bool child_of( IceWmMenuListViewItem *);
        void moveLast (IceWmMenuListViewItem *lv);
protected:
	void mousePressEvent( QMouseEvent * /*e*/ );
};

class IceWmMenuListView : public QListView
{
Q_OBJECT 

public: IceWmMenuListView (QWidget *parent=0) ;
	IceWmMenuListViewItem *root_lvi;
	void moveLast( IceWmMenuListViewItem *lv);
	QDragObject *dragObject();

protected:
    void dragEnterEvent( QDragEnterEvent * );
    void dragMoveEvent( QDragMoveEvent * );
    void dragLeaveEvent( QDragLeaveEvent * );
    void dropEvent( QDropEvent * );
    void mouseReleaseEvent ( QMouseEvent * e );
private:
	QPopupMenu *menu;
	QListViewItem *itemTaken;
	int dirty;

public slots:
	void slotAddMenuItem();
	void slotAddProgItem(int);
	void slotAddSeparatorItem();
	void slotEditItem();
	void slotDeleteItem();
	void slotCutItem();
	void slotPasteAsChildItem();
	void slotPasteAsElderBrotherItem();
	void slotPasteAsYoungerBrotherItem();
	void slotMenuRequested( QListViewItem *, const QPoint&, int);
	void slotDoubleClickedEditItem( QListViewItem *);
	void slotExitButton();
	void slotSaveButton();
	void slotmouseButtonPressed ( int button, QListViewItem * item, const QPoint & pos, int c );
};

class IceWmMenuItemDrag : public QStoredDrag {
public:
    IceWmMenuItemDrag( QListViewItem *item, QWidget * parent = 0, const char * name = 0 );

    static bool canDecode( QDragMoveEvent* e );
//    static bool canDecode( QDragMoveEvent* e );
//    static bool decode( QDropEvent* e, QString& s ); 
    static bool decode( QDropEvent* e, IceWmMenuListViewItem ** dragItem ); 
};

#endif
